#!/usr/bin/ruby -w
# frozen_string_literal: true

require "fileutils"

EXT = {
  "JPEG" => ".jpg",
  "PNG " => ".png",
  "MOD " => ".mod",
  "AIFF" => ".aiff",
  "GLUL" => ".ulx",
  "ZCOD" => ".z8",
  "IFmd" => ".xml",
  "OGGV" => ".ogg",
}

def extract(f, start)
  f.seek start, IO::SEEK_SET
  type = f.read(4)
  size = f.read(4).unpack1("N1")
  chunk = f.read(size)

  if type == "FORM" && chunk[0, 4] == "AIFF"
    chunk = "FORM#{[size].pack("N1")}#{chunk}"
    type = "AIFF"
  end

  [type, chunk]
end

ARGV.each do |blorb|
  File.open(blorb) do |f|
    raise "Not a Blorb file" if f.read(4) != "FORM"
    f.seek 8, IO::SEEK_SET
    raise "Not a Blorb file" if f.read(4) != "IFRS" || f.read(4) != "RIdx"

    f.seek 4, IO::SEEK_CUR
    nres = f.read(4).unpack1("N1")

    0.upto(nres - 1).each do
      f.seek 4, IO::SEEK_CUR
      number = f.read(4).unpack1("N1")
      start = f.read(4).unpack1("N1")
      saved = f.tell
      type, chunk = extract f, start
      dir = "extracted/#{File.basename blorb}"
      FileUtils.mkdir_p dir

      File.write(format("#{dir}/#{type.rstrip}%03d#{EXT[type]}", number), chunk)
      f.seek saved, IO::SEEK_SET
    end
  end
end
